@@ -1,3 +1,4 @@
+import warnings
 from datetime import datetime, timezone
 from typing import Dict, Callable, Any, Optional
 import copy
@@ -9,7 +10,7 @@ from UnleashClient.api import register_client
 from UnleashClient.periodic_tasks import fetch_and_load_features, aggregate_and_send_metrics
 from UnleashClient.strategies import ApplicationHostname, Default, GradualRolloutRandom, \
     GradualRolloutSessionId, GradualRolloutUserId, UserWithId, RemoteAddress, FlexibleRollout
-from UnleashClient.constants import METRIC_LAST_SENT_TIME, DISABLED_VARIATION
+from UnleashClient.constants import METRIC_LAST_SENT_TIME, DISABLED_VARIATION, ETAG
 from .utils import LOGGER
 from .deprecation_warnings import strategy_v2xx_deprecation_check
 
@@ -22,7 +23,9 @@ class UnleashClient:
                  environment: str = "default",
                  instance_id: str = "unleash-client-python",
                  refresh_interval: int = 15,
+                 refresh_jitter: Optional[int] = None,
                  metrics_interval: int = 60,
+                 metrics_jitter: Optional[int] = None,
                  disable_metrics: bool = False,
                  disable_registration: bool = False,
                  custom_headers: Optional[dict] = None,
@@ -39,7 +42,9 @@ class UnleashClient:
         :paramgit  environment: Name of the environment using the unleash client, optional & defaults to "default".
         :param instance_id: Unique identifier for unleash client instance, optional & defaults to "unleash-client-python"
         :param refresh_interval: Provisioning refresh interval in ms, optional & defaults to 15 seconds
+        :param refresh_jitter: Provisioning refresh interval jitter in s, optional & defaults to None
         :param metrics_interval: Metrics refresh interval in ms, optional & defaults to 60 seconds
+        :param metrics_jitter: Metrics refresh interval jitter in s, optional & defaults to None
         :param disable_metrics: Disables sending metrics to unleash server, optional & defaults to false.
         :param custom_headers: Default headers to send to unleash server, optional & defaults to empty.
         :param custom_options: Default requests parameters, optional & defaults to empty.
@@ -57,7 +62,9 @@ class UnleashClient:
         self.unleash_environment = environment
         self.unleash_instance_id = instance_id
         self.unleash_refresh_interval = refresh_interval
+        self.unleash_refresh_jitter = int(refresh_jitter) if refresh_jitter is not None else None
         self.unleash_metrics_interval = metrics_interval
+        self.unleash_metrics_jitter = int(metrics_jitter) if metrics_jitter is not None else None
         self.unleash_disable_metrics = disable_metrics
         self.unleash_disable_registration = disable_registration
         self.unleash_custom_headers = custom_headers
@@ -76,6 +83,7 @@ class UnleashClient:
         self.fl_job: Job = None
         self.metric_job: Job = None
         self.cache[METRIC_LAST_SENT_TIME] = datetime.now(timezone.utc)
+        self.cache[ETAG] = ''
         self.cache.sync()
 
         # Mappings
@@ -109,49 +117,65 @@ class UnleashClient:
 
         :return:
         """
-        # Setup
-        fl_args = {
-            "url": self.unleash_url,
-            "app_name": self.unleash_app_name,
-            "instance_id": self.unleash_instance_id,
-            "custom_headers": self.unleash_custom_headers,
-            "custom_options": self.unleash_custom_options,
-            "cache": self.cache,
-            "features": self.features,
-            "strategy_mapping": self.strategy_mapping,
-            "project": self.unleash_project_name
-        }
+        # Only perform initialization steps if client is not initialized.
+        if not self.is_initialized:
+            try:
+                # Setup
+                fl_args = {
+                    "url": self.unleash_url,
+                    "app_name": self.unleash_app_name,
+                    "instance_id": self.unleash_instance_id,
+                    "custom_headers": self.unleash_custom_headers,
+                    "custom_options": self.unleash_custom_options,
+                    "cache": self.cache,
+                    "features": self.features,
+                    "strategy_mapping": self.strategy_mapping,
+                    "project": self.unleash_project_name
+                }
 
-        metrics_args = {
-            "url": self.unleash_url,
-            "app_name": self.unleash_app_name,
-            "instance_id": self.unleash_instance_id,
-            "custom_headers": self.unleash_custom_headers,
-            "custom_options": self.unleash_custom_options,
-            "features": self.features,
-            "ondisk_cache": self.cache
-        }
+                metrics_args = {
+                    "url": self.unleash_url,
+                    "app_name": self.unleash_app_name,
+                    "instance_id": self.unleash_instance_id,
+                    "custom_headers": self.unleash_custom_headers,
+                    "custom_options": self.unleash_custom_options,
+                    "features": self.features,
+                    "ondisk_cache": self.cache
+                }
 
-        # Register app
-        if not self.unleash_disable_registration:
-            register_client(self.unleash_url, self.unleash_app_name, self.unleash_instance_id,
-                            self.unleash_metrics_interval, self.unleash_custom_headers,
-                            self.unleash_custom_options, self.strategy_mapping)
+                # Register app
+                if not self.unleash_disable_registration:
+                    register_client(self.unleash_url, self.unleash_app_name, self.unleash_instance_id,
+                                    self.unleash_metrics_interval, self.unleash_custom_headers,
+                                    self.unleash_custom_options, self.strategy_mapping)
 
-        fetch_and_load_features(**fl_args)
+                fetch_and_load_features(**fl_args)
 
-        # Start periodic jobs
-        self.scheduler.start()
-        self.fl_job = self.scheduler.add_job(fetch_and_load_features,
-                                             trigger=IntervalTrigger(seconds=int(self.unleash_refresh_interval)),
-                                             kwargs=fl_args)
+                # Start periodic jobs
+                self.scheduler.start()
+                self.fl_job = self.scheduler.add_job(fetch_and_load_features,
+                                                     trigger=IntervalTrigger(
+                                                         seconds=int(self.unleash_refresh_interval),
+                                                         jitter=self.unleash_refresh_jitter,
+                                                     ),
+                                                     kwargs=fl_args)
 
-        if not self.unleash_disable_metrics:
-            self.metric_job = self.scheduler.add_job(aggregate_and_send_metrics,
-                                                     trigger=IntervalTrigger(seconds=int(self.unleash_metrics_interval)),
-                                                     kwargs=metrics_args)
-
-        self.is_initialized = True
+                if not self.unleash_disable_metrics:
+                    self.metric_job = self.scheduler.add_job(aggregate_and_send_metrics,
+                                                             trigger=IntervalTrigger(
+                                                                 seconds=int(self.unleash_metrics_interval),
+                                                                 jitter=self.unleash_metrics_jitter,
+                                                             ),
+                                                             kwargs=metrics_args)
+            except Exception as excep:
+                # Log exceptions during initialization.  is_initialized will remain false.
+                LOGGER.warning("Exception during UnleashClient initialization: %s", excep)
+                raise excep
+            else:
+                # Set is_iniialized to true if no exception is encountered.
+                self.is_initialized = True
+        else:
+            warnings.warn("Attempted to initialize an Unleash Client instance that has already been initialized.")
 
     def destroy(self):
         """
