@@ -1,7 +1,7 @@
 from fcache.cache import FileCache
 from UnleashClient.api import get_feature_toggles
 from UnleashClient.loader import load_features
-from UnleashClient.constants import FEATURES_URL
+from UnleashClient.constants import FEATURES_URL, ETAG
 from UnleashClient.utils import LOGGER
 
 
@@ -14,7 +14,15 @@ def fetch_and_load_features(url: str,
                             features: dict,
                             strategy_mapping: dict,
                             project: str = None) -> None:
-    feature_provisioning = get_feature_toggles(url, app_name, instance_id, custom_headers, custom_options, project)
+    (feature_provisioning, etag) = get_feature_toggles(
+        url,
+        app_name,
+        instance_id,
+        custom_headers,
+        custom_options,
+        project,
+        cache[ETAG]
+    )
 
     if feature_provisioning:
         cache[FEATURES_URL] = feature_provisioning
@@ -22,4 +30,8 @@ def fetch_and_load_features(url: str,
     else:
         LOGGER.warning("Unable to get feature flag toggles, using cached provisioning.")
 
+    if etag:
+        cache[ETAG] = etag
+        cache.sync()
+
     load_features(cache, features, strategy_mapping)
