@@ -3,12 +3,15 @@ import mmh3  # pylint: disable=import-error
 from requests import Response
 
 LOGGER = logging.getLogger(__name__)
+LOGGER.setLevel("CRITICAL")
 
+APSCHEDULER_LOGGER = logging.getLogger("apscheduler.scheduler")
+APSCHEDULER_LOGGER.setLevel("CRITICAL")
 
 def normalized_hash(identifier: str,
                     activation_group: str,
                     normalizer: int = 100) -> int:
-    return mmh3.hash("{}:{}".format(activation_group, identifier), signed=False) % normalizer + 1
+    return mmh3.hash(f"{activation_group}:{identifier}", signed=False) % normalizer + 1
 
 
 def get_identifier(context_key_name: str, context: dict) -> str:
